SLM={init={}}
SLM.data={presets={},stations={}} --saved to system notes
SLM.presets_index = {} --which stations use preset, reconstructed based on notes data
SLM.NAME_MAX_LENGTH = 20 --max number of characters for preset name, limited by button size
SLM.NOTES = {
	main = "562368870", --spells LOADOUTS 0
	import = "562368871", --1 for Import (TODO)
	export = "562368873", --3 for Export (TODO)
	bak = "562368875", --5 for Save (TODO)
}
SLM.version = '0.4.5a'

--## Choco ##--

SLM.vars = dofile('vars.lua')
SLM.ui = dofile('ui.lua')

SLM.RESERVED_NAMES = {['ACTIVE SHIP']=true,UNASSIGNED=true,['NO SHIP']=true,['#ERROR']=true}

--assigns preset to station -- validate parameters are valid before executing this function
local function DoStationAssignment(name, stationid, i)
	--console_print('Doing Station Assignments') --DEBUG
	SLM.ClearStationIndex(stationid, i) --remove old loadout in slot i
			
	--link name of loadout with station
	SLM.data.stations[stationid] = SLM.data.stations[stationid] or {}
	SLM.data.stations[stationid][i] = name			
	
	--save entry in names index table
	SLM.presets_index[name] = SLM.presets_index[name] or {}
	SLM.presets_index[name][stationid] = SLM.presets_index[name][stationid] or {}
	SLM.presets_index[name][stationid][i] = true
	
	SLM.replace_buyback_titles()
	SLM.replace_station_titles()
end

local function DoDefaultAssignment(loadout, i)
	--console_print('Doing Default Assignments') --DEBUG
	--TODO verify loadout and i are valid
	
	LoadShipPresets() --restore defaults from config
	ShipPresets[i] = loadout --replace default
	SaveShipPresets() --save defaults to config
end

--replace default preset in config file with one specified or else use the one from station
local function ReplacePreset(i, loadout)
	local stationid = GetStationLocation()
	local name = SLM.data.stations[stationid] and SLM.data.stations[stationid][i]
	local station_loadout = SLM.data.presets[name]
	
	loadout = loadout or station_loadout --if loadout not specified use loadout for the current station
	
	if loadout then
		ShipPresets[i] = loadout
		SaveShipPresets(i) --overwrite default loadout in config
		return true
	else return false end
end

--original actions for buy preset buttons (default presets)
local default_actions = {
	StationEquipmentBuyShipTab[1][3][6][1][1][1].action,
	StationEquipmentBuyShipTab[1][3][6][1][1][2].action,
	StationEquipmentBuyShipTab[1][3][6][1][2][1].action,
	StationEquipmentBuyShipTab[1][3][6][1][2][2].action,
}
--IUP.buttons for buy preset buttons
local default_buttons = {
	StationEquipmentBuyShipTab[1][3][6][1][1][1],
	StationEquipmentBuyShipTab[1][3][6][1][1][2],
	StationEquipmentBuyShipTab[1][3][6][1][2][1],
	StationEquipmentBuyShipTab[1][3][6][1][2][2],
}
--original actions for set presets buttons
local preset_actions = {
	StationEquipmentManageShipSelectionTab[3][5][1][2][1].action,
	StationEquipmentManageShipSelectionTab[3][5][1][2][2].action,
	StationEquipmentManageShipSelectionTab[3][5][1][3][1].action,
	StationEquipmentManageShipSelectionTab[3][5][1][3][2].action,
}
--IUP.buttons for set presets buttons
local preset_buttons = {
	StationEquipmentManageShipSelectionTab[3][5][1][2][1],
	StationEquipmentManageShipSelectionTab[3][5][1][2][2],
	StationEquipmentManageShipSelectionTab[3][5][1][3][1],
	StationEquipmentManageShipSelectionTab[3][5][1][3][2],
}
--original actions for buyback buttons


--Override LoadShipPresets function to use custom buyback button titles
local load_presets_old = LoadShipPresets
function LoadShipPresets(...)
	--console_print('Load Ship Presets') --DEBUG
	local val = load_presets_old(...) --call original function
	SLM.replace_station_titles() --override buttons
	
	return val
end

--Override SaveShipPresets function to use custom buyback button titles
local save_presets_old = SaveShipPresets
function SaveShipPresets(...)
	--console_print('SaveShipPresets') --DEBUG
	local val = save_presets_old(...) --call original function
	SLM.replace_station_titles() --override buttons
	
	return val
end

local buyback_dlg
local buyback_default_buttons = {} --original buttons on buyback dlg
local buyback_default_actions = {} --actions for original buyback buttons
local buyback_preset_actions = {} --actions for new buyback buttons

--Override MakeBuyBackQuestionDlg function
local make_buyback_old = MakeBuyBackQuestionDlg
function MakeBuyBackQuestionDlg(...)
	local dlg = make_buyback_old(...)
	
	--console_print('Making Buyback Dialog') --DEBUG
	
	buyback_default_actions = {
		dlg[1][1][1][3][1].action,
		dlg[1][1][1][3][2].action,
		dlg[1][1][1][3][3].action,
		dlg[1][1][1][3][4].action,
	}
	
	buyback_default_buttons = {
		dlg[1][1][1][3][1],
		dlg[1][1][1][3][2],
		dlg[1][1][1][3][3],
		dlg[1][1][1][3][4],
	}
	
	for i=1,4 do
		buyback_preset_actions[i] = function()
			local default_loadout = ShipPresets[i] --save reference to default loadout
			local isOverride = ReplacePreset(i) --override default loadout in config file with station's loadout
			local val = buyback_default_actions[i]() --make purchase using overriden loadout
			if isOverride then ReplacePreset(i, default_loadout) end --restore default loadout in config file
			
			HideDialog(dlg)
			return val
		end
	end
	
	local isReload = true --buyback_dlg==nil 
	buyback_dlg = dlg
	
	--if PlayerInStation() then SLM.replace_button_titles() end --BAD!!!
	if isReload then SLM.replace_buyback_titles() end
	return dlg
end

--New action to perform when clicking buyback and set preset buttons
for i=1,4 do
	default_buttons[i].action = function(...)
		local default_loadout = ShipPresets[i] --save reference to default loadout
		local isOverride = ReplacePreset(i) --override default loadout in config file with station's loadout
		local val = default_actions[i](...) --make purchase using overriden loadout
		if isOverride then ReplacePreset(i, default_loadout) end --restore default loadout in config file
		
		return val
	end
	
	preset_buttons[i].action = function(self)
		local set_preset_cb = function(name)
			if name~="DEFAULT" then
				SLM:save{i, name}
			else preset_actions[i](self) end --save as default (original button function)
		end
		
		local rename_dlg = SLM.ui.MakeRenameDialog(set_preset_cb)
		ShowDialog(rename_dlg,iup.CENTER,iup.CENTER)
	end
end

function SLM.init:PLAYER_ENTERED_GAME(event, data)
	--console_print('Initializing SLM') --DEBUG
	SLM.data = unspickle(LoadSystemNotes(SLM.NOTES.main) or "") or {stations={},presets={}}
	SLM.data.stations = SLM.data.stations or {} --protect against corrupted notes file
	SLM.data.presets = SLM.data.presets or {} --protect against corrupted notes file
	
	--create presets index table from notes data
	for stationid,presets in pairs(SLM.data.stations) do
		for i,name in pairs(presets) do --i=1,4
			if name then
				SLM.presets_index[name] = SLM.presets_index[name] or {}
				SLM.presets_index[name][stationid] = SLM.presets_index[name][stationid] or {}
				SLM.presets_index[name][stationid][i] = true
			end
		end
	end
	
	--console_print('Loaded Presets From System Notes') --DEBUG
	
	SLM.replace_station_titles() --NOT redundant, sometimes event occurs after LoadShipPresets()
end

--Override buyback button titles when entering station
function SLM.enter_station(event, data)
	--console_print('entering station') --DEBUG
	
	--LoadShipPresets()
	SLM.replace_buyback_titles() --must call before replacing station titles (which recreates buyback dialog)
	SLM.replace_station_titles()
end

function SLM.replace_station_titles(titles, isUseColor)
	--console_print('Replacing Station Titles') --DEBUG
	local use_color = isUseColor or (titles==nil and true or false) --only use color when station presets are used or isUseColor flag set
	titles = titles or {}
	
	if PlayerInStation() then --TODO: safety check to verify not in capship?
		local stationid = GetStationLocation()
		local station_presets = SLM.data.stations[stationid] and SLM.data.stations[stationid] or {}
		
		load_presets_old() --load default presets first to clear existing text
		
		for i=1,4 do
			titles[i] = titles[i] or station_presets[i]
			titles[i] = titles[i] and titles[i]:len() > SLM.NAME_MAX_LENGTH and titles[i]:sub(1,SLM.NAME_MAX_LENGTH-3)..'...' or titles[i] --truncate
			
			if titles[i] and use_color then
				--calculate color
				local color = SLM.GetLoadoutColor(station_presets[i] and SLM.data.presets[ station_presets[i] ]) or ''
				default_buttons[i].title = color..titles[i]
			end
		end
		
		--iup.Refresh(StationEquipmentBuyShipTab) --neccessary?
	end
end

function SLM.replace_buyback_titles(titles, isUseColor)
	--console_print('Replacing Buyback Titles') --DEBUG
	local use_color = isUseColor or (titles==nil and true or false) --only use color when station presets are used or isUseColor flag set
	titles = titles or {}
	
	if not buyback_dlg then --[[console_print('buyback empty');]] return end
	
	if PlayerInStation() then --TODO: safety check to verify not in capship?
		local stationid = GetStationLocation()
		local station_presets = SLM.data.stations[stationid] and SLM.data.stations[stationid] or {}
		
		local new_buttons = {}
		for i=1,4 do
			titles[i] = titles[i] or station_presets[i]
			titles[i] = titles[i] and titles[i]:len() > SLM.NAME_MAX_LENGTH and titles[i]:sub(1,SLM.NAME_MAX_LENGTH-3)..'...' or titles[i] --truncate
			
			if not titles[i] then --[[console_print('System Notes not yet loaded');]] return end
			
			if titles[i] and use_color then
				--calculate color
				local color = SLM.GetLoadoutColor(station_presets[i] and SLM.data.presets[ station_presets[i] ]) or ''		
				
				new_buttons[i] = iup.stationbutton{title=color..titles[i]}
				new_buttons[i].action=buyback_preset_actions[i]
			else new_buttons[i] = iup.fill{size="100"} end
		end
		
		if next(titles) then --at least one entry in titles
			new_buttons.gap=6
			local new_button_box = iup.hbox(new_buttons)
			
			--remove existing buttons if exist
			if buyback_dlg and buyback_dlg[1][1][1][4] then iup.Detach(buyback_dlg[1][1][1][4]) end
			
			iup.Append(buyback_dlg[1][1][1], new_button_box)
			buyback_dlg[1][1][1][4] = new_button_box
			--iup.Map(iup.GetDialog(new_button_box))
			iup.Map(buyback_dlg)
    		iup.Refresh(buyback_dlg)
		end
		
		--console_print('new titles appended') --DEBUG
	else --[[console_print('not in station')]] end
end

SLM.inputs = {save=true,assign=true,remove=true,delete=true,list=true,buy=true,rename=true,help=true,debug=false} --TODO: add export/import
function SLM:input(args)
	args = args or {}
	local command = args[1] and args[1]:lower() or nil--'help'
	
	if SLM.inputs[command] then
		table.remove(args, 1)
		SLM[command](self, args)
	else SLM.ui:Show() end --Open GUI by default
end

--saves current ship to specified loadout slot at this station
function SLM:save(args)
	if args and args[1] then
		local isDefault,i = string.match(args[1], '^(%u*)%s?(%d+)')
		i = tonumber(i)
		isDefault = isDefault=="DEFAULT"
		local name = args[2]
		local loadout = args[3]
		local stationid = GetStationLocation()
		
		--Safety checks
		if not (i and i>=0 and i<=4) then print'\127ffffffSlot number must be 0-4'; return end
		if math.floor(i) ~= i then print'\127ffffffSlot number must be an integer'; return end
		if not name then print'\127ffffffYou must specify a name for the preset to save at this station'; return end
		if name and string.len(name) > SLM.NAME_MAX_LENGTH then print('\127ffffffName must not be longer than '..SLM.NAME_MAX_LENGTH..' characters'); return end
		--TODO: more detailed verification of name
		if type(loadout)~='nil' and type(loadout)~='table' then print'invalid loadout'; return end
		if stationid == nil then print'\127ffffffYou must dock at a station first'; return end
		
		if not SLM.data.presets[name] then
			loadout = loadout or SaveShipLoadout(GetActiveShipID()) --TODO: uses active ship, use selected ship instead?
			
			if not isDefault then
				SLM.data.presets[name] = loadout --save loadout in data
				if i>0 then DoStationAssignment(name, stationid, i) end --slot 0 saves new loadout but does not assign to the station
				
				SaveSystemNotes(spickle(SLM.data), SLM.NOTES.main)
			else DoDefaultAssignment(loadout, i) end
			
			print('\127ffffffLoadout for slot '..i..' saved')
			return true
		else print('\127ffffff'..name..' has already been assigned. To reuse '..name..' at this station do: /SLM assign '..i..' '..name); return end
	else print'\127ffffffYou must specify a slot (0-4) to save the current loadout to: e.g. /SSL save 1'; return end
end

--assigns previously saved loadout to this station
function SLM:assign(args)
	if args and args[1] then
		local isDefault,i = string.match(args[1], '^(%u*)%s?(%d+)')
		i = tonumber(i)
		isDefault = isDefault=="DEFAULT"
		local name = args[2]
		local stationid = GetStationLocation()
		
		--safety checks
		if not (i and i>=1 and i<=4) then print'\127ffffffSlot number must be 1-4'; return end
		if math.floor(i) ~= i then print'\127ffffffSlot number must be an integer'; return end
		if not name then print'\127ffffffYou must specify the name of the loadout to assign at this station'; return end
		if name and string.len(name) > SLM.NAME_MAX_LENGTH then print('\127ffffffName must not be longer than '..SLM.NAME_MAX_LENGTH..' characters'); return end
		if stationid == nil then print'\127ffffffYou must dock at a station first'; return end
		
		if SLM.data.presets[name] ~= nil then
			if not isDefault then
				DoStationAssignment(name, stationid, i)
				SaveSystemNotes(spickle(SLM.data), SLM.NOTES.main)
			else DoDefaultAssignment(SLM.data.presets[name], i) end
			
			print('\127ffffffLoadout for slot '..i..' saved')
			return true
		else print('\127ffffffLoadout does not exist: '..name) end
	else print'\127ffffffYou must specify a slot (1-4) to save the specified loadout to: e.g. /SSL assign 1 NAME' end
end

--removes loadout for specified slot from current station
function SLM:remove(args)
	if args and args[1] then
		local i = tonumber(args[1])
		local stationid = GetStationLocation()
		
		if not (i>=1 and i<=4) then print'\127ffffffSlot number must be 1-4'; return end
		if math.floor(i) ~= i then print'\127ffffffSlot number must be an integer'; return end
		if stationid == nil then print'\127ffffffYou must dock at a station first'; return end
		
		if SLM.ClearStationIndex(stationid, i) then
			SaveSystemNotes(spickle(SLM.data), SLM.NOTES.main)
			LoadShipPresets()
			print('\127ffffffLoadout for slot '..i..' removed')
			
			return true
		else print('\127ffffffNo loadout data found for slot '..i) end
	else print'\127ffffffYou must specify a slot (1-4) to remove: e.g. /SSL remove 1' end
end

--deletes specified loadout and removes from all stations
function SLM:delete(args)
	local name = args and args[1]
	
	if not name then print'\127ffffffYou must specify the name of the loadout to delete'; return end
	if string.len(name) > SLM.NAME_MAX_LENGTH then print('\127ffffffName must not be longer than '..SLM.NAME_MAX_LENGTH..' characters'); return end
	if not SLM.data.presets[name] then print('\127ffffffloadout '..name..' not found'); return end
	
	if SLM.presets_index[name] then --TODO redundant
		--remove all instances of loadout:name in SLM.data.stations 
		for stationid,station_presets in pairs(SLM.presets_index[name]) do
			for i,_ in pairs(station_presets) do
				if SLM.data.stations[stationid][i]==name then SLM.data.stations[stationid][i]=nil end --verify preset is one to remove and remove it
			end
			
			if table.maxn(SLM.data.stations[stationid])==0 then SLM.data.stations[stationid]=nil end --remove station entry if all presets removed
		end
		
		SLM.presets_index[name] = nil
	end
	
	SLM.data.presets[name] = nil --delete preset data
	
	SaveSystemNotes(spickle(SLM.data), SLM.NOTES.main)
	
	SLM.replace_buyback_titles()
	SLM.replace_station_titles()
	
	print('\127ffffffLoadout '..name..' deleted')
	return true
end

function SLM:list(args)
	args = args or {}
	local command = args[1] and args[1]:lower()
	
	if command == 'all' then
		print'\127ffffffAll named loadout presets:'
		for name,loadout in pairs(SLM.data.presets) do
			local availability = SLM.checkAvailability(loadout) --TODO:NYI
			local color = SLM.GetLoadoutColor(loadout) or '\127ffffff'
			local assigned = SLM.presets_index[name] and '' or '*' --preface with * if preset not assigned at a station
			print(color..assigned..name)
		end
		if not next(SLM.data.presets) then print'\127ffffffNo presets found'; return end
	elseif command == 'defaults' then
		print'\127ffffffDefault loadout presets:'
		for k,v in pairs(ShipPresets) do
			print('\127ffffff'..k..': '..(InvManager.GetInventoryItemParameter(v.ship, 'Name') or 'NONE'))
		end
	else
		local stationid = GetStationLocation()
		if SLM.data.stations[stationid] then
			print'\127ffffffLocal station loadout presets:'
			for i,name in pairs(SLM.data.stations[stationid]) do
				local loadout = SLM.data.presets[name]
				local color = SLM.GetLoadoutColor(loadout) or ''
				print('\127ffffff'..i..': '..color..(name or '*'..InvManager.GetInventoryItemParameter(loadout.ship, 'Name')))
			end
		else print'No loadouts assigned at this station. Using universe-wide defaults'; return end
	end
end

--args[1] can be a number (1-4) indicating the slot number to buy or a string indicating the name of the loadout to buy
function SLM:buy(args)
	--TODO: Check availability of loadout before purchase when specifying by name
	if args and args[1] then
		local name = args[1] --ignore unless i is nil
		local i = tonumber(args[1]) --nil if not a number
		local stationid = GetStationLocation()
		
		if stationid==nil then print'\127ffffffYou must dock at a station first'; return end
		
		if type(i)=='number' then
			if not (i>=1 and i<=4) then print'\127ffffffSlot number must be 1-4'; return end
			if math.floor(i) ~= i then print'\127ffffffSlot number must be an integer'; return end
			
			local name = SLM.data.stations[stationid] and SLM.data.stations[stationid][i]
			--loadout = SLM.data.presets[name]
		--else loadout = SLM.data.presets[name] end
		end
		
		local loadout = SLM.GetLoadout(name)
		
		i = i or 1 --use slot 1 to save loadout if not specified
		if loadout then
			--PurchaseShipLoadout(nil, loadout)
			local default_loadout = ShipPresets[i] --save reference to default loadout
			local isOverride = ReplacePreset(i, loadout) --override default loadout with loadout
			default_actions[i](default_buttons[i]) --make purchase using overriden loadout
			if isOverride then ReplacePreset(i, default_loadout) end --restore default loadout
			
			return true
		else print'\127ffffffNo loadout data found'; return end
	else print'\127ffffffYou must specify a slot (1-4) to buy: e.g. /SSL buy 1'; return end
end

function SLM:rename(args)
	if args and args[1] and args[2] then
		local orig_name = args[1]
		local new_name = args[2]
		
		if #args>2 then print'127ffffffSpecify only 2 parameters. If a name contains spaces then use quotes'; return end
		--TODO:Validate new_name is valid
		if SLM.data.presets[new_name] then print('\127ffffffThere is already a loadout named: '..new_name); return end
		
		for stationid,station_presets in pairs(SLM.data.stations) do
			for i,name in pairs(station_presets) do
				if name==orig_name then station_presets[i] = new_name end
			end
		end
		
		--swap old with new and delete old data --TODO: optimize
		SLM.data.presets[new_name] = SLM.data.presets[orig_name]
		SLM.data.presets[orig_name] = nil
		SLM.presets_index[new_name] = SLM.presets_index[orig_name]
		SLM.presets_index[orig_name] = nil
		
		SaveSystemNotes(spickle(SLM.data), SLM.NOTES.main)
		
		SLM.replace_buyback_titles()
		SLM.replace_station_titles()
	
		print('\127ffffff'..orig_name..' renamed to '..new_name)
		return true
	else return SLM:help{'rename'} end
end

function SLM.checkAvailability(loadout)
	return true --TODO
	
	--[=[local stationid = GetStationLocation()
	station_ships=GetStationShipList(stationid) --list of ships in storage
	station_addons=GetStationAddonList(stationid) --list of addons in storage
	
	local purchase_itemtypeids = {} --table with itemtypeids used as keys for all ships and addons sold by station
	local item_types = {battery=true,lightweapon=true,heavyweapon=true,turretweapon=true,ship=true} --valid item types, exclude commodities
	for i=1,GetNumStationMerch() do
		local item = GetStationMerchInfo(i)
		if item_types[item.type] then purchase_itemtypeids[GetInventoryItemType(item.itemid)]=true end
	end
	
	local storage_itemtypeids = {} --table with itemtypeids used as keys for all ships and addons in storage
	for _,shipid in ipairs(station_ships) do storage_itemtypeids[GetInventoryItemType(shipid)]=true end
	for _,itemid in ipairs(station_addons) do storage_itemtypeids[GetInventoryItemType(shipid)]=true end
	
	if not storage_itemtypeids[inventoryloadout.ship] then return false end --ship not in storage
	for i,itemtypeid in ipairs(loadout.ports)
		if not storage_itemtypeids[itemtypeid] then return false end --addon not in storage
	end
	
	return true]=]
end

--remove loadout i from stationid (does not save config)
function SLM.ClearStationIndex(stationid, i) --TODO make local function for saftey
	if SLM.data.stations[stationid] and SLM.data.stations[stationid][i] then
		local name = SLM.data.stations[stationid][i] --name of preset to remove
		
		--remove entry from station loadouts
		SLM.data.stations[stationid][i] = nil
		if not next(SLM.data.stations[stationid]) then SLM.data.stations[stationid] = nil end --remove table if empty
		
		--remove entry from names index table
		if SLM.presets_index[name] and SLM.presets_index[name][stationid] then
			SLM.presets_index[name][stationid][i] = nil --remove preset from station
			if not next(SLM.presets_index[name][stationid]) then
				SLM.presets_index[name][stationid] = nil --remove station entry from list if has no saved presets
				if not next(SLM.presets_index[name]) then SLM.presets_index[name] = nil end --remove entry for name if not used in any other stations
			end
		end
		
		return true
	else return false end		
end

--returns escaped string with ship color from loadout
function SLM.GetLoadoutColor(loadout)
	if loadout and loadout.shipcolor then
		local rgb = ShipPalette_string[loadout.shipcolor+1] --index shift: ShipPallette is 1-256, not 0-255
		return SLM.vars.isUseColor and rgbtohex(rgb)
	else return nil end
end

function SLM.GetLoadout(name)
	--return loadout for saved preset
	if SLM.data.presets[name]~=nil then return SLM.data.presets[name] end
	
	--return default loadout: name="DEFAULT N"
	local default_num = tonumber(name:match('^DEFAULT (%d+)$'))
	if default_num and default_num>=1 and default_num<=4 then return ShipPresets[default_num] end
	
	--return loadout of active ship: name="ACTIVE SHIP"
	if name=="ACTIVE SHIP" then return SaveShipLoadout(GetActiveShipID()) end
	
	return nil --invalid name specified
end

--verify name (string) is a valid preset name. If invalid return false and the error message, return true if valid
function SLM.VerifyName(name)
	if not name or name=='' then return false, "Name must contain at least one letter" end
	if tonumber(name) then return false, "Name must contain at least one non-numeric char" end
	if name:len() > 20 then return false, "Name cannot be longer than 20 characters" end
	if name:match('^DEFAULT') then return false, "Name cannot begin with DEFAULT" end
	if SLM.RESERVED_NAMES[name] then return false, "That name is reserved and cannot be used" end
	if SLM.data.presets[name] then return false, "There is already a preset with that name" end
		
	return true
end

function SLM.GetTruncatedShipName(loadout)
	if type(loadout)~='table' then return end
	
	local ship_name = InvManager.GetInventoryItemParameter(loadout.ship, 'Name')
	ship_name = ship_name and ship_name:len() > SLM.NAME_MAX_LENGTH-2 and ship_name:sub(1,SLM.NAME_MAX_LENGTH-5)..'...' or ship_name --truncate
	
	return ship_name
end

function SLM:debug(args)
	--[=[local new_dlg = SLM.ui.MakeRenameDialog(function() print('ok!') end)
	console_print(dotest.traverse(new_dlg))
	local new_label = iup.label{title='L1'}
	iup.Append(new_dlg[1][2][2][1], new_label)
	new_dlg[1][2][2][1][6] = new_label
	--iup.Map(new_dlg)
	ShowDialog(new_dlg,iup.CENTER,iup.CENTER)
	console_print(dotest.traverse(iup.GetDialog(new_label)))
	--iup.Detach(new_dlg[1][2][2][1][6])
	iup.Detach(new_label)
	if new_label then print'yes' else print'no' end
	iup.Append(new_dlg[1][2][2][1], iup.label{title='L2'})
	iup.Map(new_dlg)
	iup.Refresh(new_dlg)
	console_print(dotest.traverse(new_dlg))]=]
end

function SLM:help(args)
	print('\127ffffffStation Loadouts Manager v'..SLM.version..'  Available commands are:'..
	'\n\t/SLM - Opens the GUI'..
	'\n\t/SLM save \1270070c0N\127o \127808080[\127ffff00Name\127808080]\127ffffff - Saves the current ship\'s loadout to slot \1270070c0N\127o at the current station (1-4)'..
	'\n\t\tOptional parameter \127ffff00Name\127o assigns specified name to loadout for easy reuse later'..
	'\n\t/SLM assign \1270070c0N\127o \127ffff00Name\127o - Saves previously saved loadout with \127ffff00Name\127o to slot \1270070c0N\127o at this station'..
	'\n\t/SLM remove \1270070c0N\127o - Removes loadout in slot N from the current station (universe-wide default will be used)'..
	'\n\t/SLM list \127808080[\127ff00ffall\127808080/\12700ffffdefaults\127808080]\127ffffff - Display loadouts for the current station'..
	'\n\t\tOptional parameter \127ff00ffall\127o lists every named loadout, \12700ffffdefaults\127o lists universe-wide defaults'..
	'\n\t/SLM buy \127ffff00Name\127o - Purchase ship with loadout \127ffff00Name\127o')
end

RegisterUserCommand('slm',SLM.input)
RegisterEvent(SLM.enter_station, 'ENTERED_STATION')
RegisterEvent(SLM.init, 'PLAYER_ENTERED_GAME')