local ui = {}
--local vars = dofile('vars.lua')
ui.DEFAULTS_COLOR = '\127a0a0a0'
ui.MAX_NUM_PORTS = 7

--* Begin UI *--

-- Main GUI --

ui.element = {} --iup controls
ui.pane = {} --groupings of iup controls
ui.element.name_label = iup.label{title="Preset Name: ", expand="No"}
ui.element.name = iup.label{title="", expand="Horizontal"}
ui.element.ship_label = iup.label{title="Ship: ", expand="No"}
ui.element.ship = iup.label{title="", expand="Horizontal"}

ui.element.addons_label = iup.label{title="Addons:", expand="No"}
ui.element.addons = {}
for i=1,ui.MAX_NUM_PORTS do ui.element.addons[i] = iup.label{title="none", expand="Horizontal"} end

ui.element.sector_label = iup.label{title=""}
ui.element.station_label = iup.label{title=""}

ui.element.rename_button = iup.stationbutton{title="Rename"}
ui.element.delete_button = iup.stationbutton{title="Delete Preset"}
--ui.element.save_color_button = iup.stationbutton{title="Save Color"} --TODO
ui.element.buy_button = iup.stationbutton{title="Buy Preset"}
ui.element.close_button = iup.stationbutton{title="Close"}
ui.element.help_button = iup.stationbutton{title="Help"}

ui.element.default_buttons = {}
ui.element.default_labels = {}
ui.element.preset_buttons = {}
ui.element.preset_labels = {}
ui.pane.preset_buttons = iup.hbox{gap=2}
for i=1,4 do
	ui.element.default_buttons[i] = iup.stationbutton{title="Assign Default "..i, size="180x"}
	ui.element.default_labels[i] = iup.label{title="", size="180x", alignment="ACENTER"}
	ui.element.preset_buttons[i] = iup.stationbutton{title="Assign Preset "..i, size="180x"}
	ui.element.preset_labels[i] = iup.label{title="", size="180x", alignment="ACENTER"}
	
	iup.Append(ui.pane.preset_buttons, iup.vbox{ui.element.preset_buttons[i], ui.element.preset_labels[i], ui.element.default_buttons[i], ui.element.default_labels[i]})
end

ui.element.preset_list = iup.pdasubsubsublist{value=0, size="240x360"}




local function ShowHelp()
	StationHelpDialog:Open(
[[You can manage your list of saved ship presets from this dialog window.

The list on the right lists every saved preset you've made. Select a preset to see info about which ship is used and assigned addons on the left side of the dialog.

The buttons apply to the preset selected from the list:
*Rename: Allows you to change the name of an existing preset at all stations where it is used (20 characters max)
*Delete Preset: Removes the selected preset and deletes it from all stations where it is currently assigned (default preset will be used instead). Default presets, as well as your current preset cannot be deleted.
*Buy Preset: Purchases the selected preset at the current station and makes it your active ship.

You can use the buttons along the bottom to assign one of your existing presets to the current station. The 'Assign Preset #' button will assign the preset to the current station in slot #. You can also assign it as a default loadout to all stations with the 'Assign Default #' button.
]])
end

ui.element.help_button.action=ShowHelp


ui.pane.preset_info = iup.vbox{
	iup.hbox{ui.element.name_label, ui.element.name},
	iup.hbox{ui.element.ship_label, ui.element.ship},
	iup.fill{size=20},
	ui.element.addons_label,
	iup.hbox{iup.vbox(ui.element.addons),iup.fill{}}, --TODO add color picker
	iup.fill{size=120},
}

local frame = iup.vbox{
	iup.hbox{iup.fill{}, iup.label{title="Ship Loadouts Manager",font=Font.H3}, iup.fill{}},
	iup.hbox{iup.fill{}, iup.label{title="Version "..SLM.version}}, --TODO: remove global use
	iup.hbox{ui.pane.preset_info, ui.element.preset_list, gap=8},
	iup.hbox{ui.element.sector_label, ui.element.station_label, iup.fill{}, ui.element.rename_button, ui.element.delete_button, iup.fill{size=12}, ui.element.buy_button},
	iup.fill{size=6},
	iup.hbox{ui.pane.preset_buttons, iup.fill{size=4}, iup.vbox{iup.fill{}, ui.element.help_button, ui.element.close_button, gap=4}},
	alignment="ALEFT",
}


-- Create Dialog --

ui.dialog = iup.dialog{
	iup.vbox{
		iup.fill{},
		iup.hbox{
			iup.fill{},
			iup.pdarootframe{
			--iup.stationsubframe{
				frame,
				expand="NO",
				gap=8,
			},
			iup.fill{},
		},
		iup.fill{},
	},
	BGCOLOR = "0 0 0 160 *",
	FULLSCREEN="YES",
	RESIZE="NO",
	BORDER="NO",
	MENUBOX="YES",
	TOPMOST="YES",
	DEFAULTESC=ui.element.close_button,
	DEFAULTENTER=ui.element.close_button,
}
ui.dialog:map()

-- Display Dialog --

function ui:Show()
	if not ui.dialog then ui:Init() end
	
	--TODO: reset controls based on vars settings
	
	ui.LoadPresets()

	ShowDialog(ui.dialog,iup.CENTER,iup.CENTER)
	--iup.Map(ui.dialog)
	iup.Refresh(ui.dialog)
end

function ui:Hide()
	HideDialog(ui.dialog)
end

function ui.element.rename_button:action()
	local preset_name = filter_colorcodes(ui.GetListSelected(ui.element.preset_list))
	
	if preset_name~=nil then
		local rename_cb = function(name)
			if SLM:rename{preset_name, name} then
				--Regenerate Preset button text --TODO: only update text that changed
				if PlayerInStation() then --TODO: safety check to verify not in capship?
					local stationid = GetStationLocation()
					local station_presets = SLM.data.stations[stationid] and SLM.data.stations[stationid] or {}
		
					for i=1,4 do
						if station_presets[i] then
							local color = SLM.GetLoadoutColor(SLM.data.presets[ station_presets[i] ]) or ''
							ui.element.preset_labels[i].title = color..station_presets[i]
						else ui.element.preset_labels[i].title = "UNASSIGNED" end
					end
				end
				
				--update name in list
				local list_sel = ui.element.preset_list.vo_listsel
				local color = SLM.GetLoadoutColor(SLM.data.presets[name]) or '\127ffffff'
				ui.element.preset_list[list_sel] = color..name --change name in list
				ui.UpdateListSelection() --update loadout info panel
				iup.Refresh(ui.dialog)
			end
		end
		
		local rename_dlg = ui.MakeRenameDialog(rename_cb, false)
		ShowDialog(rename_dlg,iup.CENTER,iup.CENTER)
	end
end

function ui.element.delete_button:action()
	local function ok_cb()
		local preset_name = filter_colorcodes(ui.GetListSelected(ui.element.preset_list))
		
		if preset_name~=nil then
			--ui:Hide()
			if SLM:delete{preset_name} then
				ui.LoadPresets() --TODO: More elegant method
				
				--[[ (redundant with LoadPresets)
				--update list
				local item_num = ui.element.preset_list.vo_listsel
				ui.RemoveListItem(ui.element.preset_list, item_num) --remove deleted entry
				ui.element.preset_list.value = item_num-1 --select entry above
				ui.UpdateListSelection() --update loadout info panel
				
				iup.Refresh(ui.dialog)]]
			end
		end
		
		HideDialog(QuestionDialog)
	end

	local function cancel_cb()
		HideDialog(QuestionDialog)
	end

	QuestionDialog:SetMessage("Are you sure you want to delete this loadout?\nIt will be removed from ALL stations", "Ok", ok_cb, "Cancel", cancel_cb)
	ShowDialog(QuestionDialog)
end

function ui.element.buy_button:action()
	--TODO check if ship/addons available at station
	local preset_name = filter_colorcodes(ui.GetListSelected(ui.element.preset_list))
	
	--if SLM.data.presets[preset_name]~=nil then
	if SLM.GetLoadout(preset_name) then
		ui:Hide()
		SLM:buy{preset_name}
	end
end

--Assign presets/defaults button actions
for i=1,4 do
	ui.element.preset_buttons[i].action = function()
		local preset_name = filter_colorcodes(ui.GetListSelected(ui.element.preset_list))
		local loadout = SLM.GetLoadout(preset_name)
		local color = SLM.GetLoadoutColor(loadout) or ''
		
		if SLM.data.presets[preset_name]~=nil then
			if SLM:assign{i, preset_name} then
				ui.element.preset_labels[i].title = color..(preset_name or "#ERROR")
				iup.Refresh(ui.dialog)
			end
		else
			local assign_preset_cb = function(name)
				if SLM:save{i, name, loadout} then
					ui.element.preset_labels[i].title = color..(name or "#ERROR")
					ui.LoadPresets() --to add to list --TODO: more elegant method
					iup.Refresh(ui.dialog)
				end
			end
			
			local rename_dlg = ui.MakeRenameDialog(assign_preset_cb, false)
			ShowDialog(rename_dlg,iup.CENTER,iup.CENTER)
		end
		
		iup.Refresh(ui.dialog)
	end
	
	ui.element.default_buttons[i].action = function()
		local preset_name = filter_colorcodes(ui.GetListSelected(ui.element.preset_list))
		local loadout = SLM.GetLoadout(preset_name)
		
		if SLM.data.presets[preset_name]~=nil then
			if SLM:assign{"DEFAULT"..i, preset_name} then
				ui.element.default_labels[i].title = ui.DEFAULTS_COLOR..(SLM.GetTruncatedShipName(loadout) or "#ERROR")
				iup.Refresh(ui.dialog)
			end
		else
			if SLM:save{"DEFAULT"..i, preset_name, loadout} then
				ui.element.default_labels[i].title = ui.DEFAULTS_COLOR..(SLM.GetTruncatedShipName(loadout) or "#ERROR")
				iup.Refresh(ui.dialog)
			end
		end	
	end
end

function ui.element.preset_list:action(item, n, sel)
	if sel==1 then ui.UpdateListSelection(item, n) end
end

function ui.UpdateListSelection(item, n)
	item = item or ui.GetListSelected(ui.element.preset_list)
	n = n or ui.element.preset_list.value
	
	ui.element.name.title = item
	local preset_name = filter_colorcodes(item)
	local loadout = SLM.GetLoadout(preset_name) --SLM.data.presets[preset_name]
	
	if loadout then
		ui.element.ship.title = InvManager.GetInventoryItemParameter(loadout.ship, 'Name')
		
		
		--TODO: make more elegant
		--assign addon names
		for i=1,ui.MAX_NUM_PORTS do ui.element.addons[i].title = 'none' end
		for i,itemtypeid in ipairs(loadout.ports) do
			local addon_name = InvManager.GetInventoryItemParameter(itemtypeid, 'Name') or 'none'
			ui.element.addons[i].title = addon_name
		end
		
		for i=1,4 do
			ui.element.preset_buttons[i].active = PlayerInStation() and "YES" or "NO"
			ui.element.default_buttons[i].active = "YES"
		end
		ui.element.buy_button.active = "YES"
	else --e.g. no active ship
		ui.element.ship.title = "No Ship"
		for i=1,ui.MAX_NUM_PORTS do ui.element.addons[i].title = 'none' end
		
		--disable buttons because loadout is invalid
		for i=1,4 do
			ui.element.preset_buttons[i].active = "NO"
			ui.element.default_buttons[i].active = "NO"
		end
		ui.element.buy_button.active = "NO"
	end
	
	--disable rename/delete buttons for special loadouts
	if SLM.data.presets[preset_name] then
		ui.element.rename_button.active = "YES"
		ui.element.delete_button.active = "YES"
	else
		ui.element.rename_button.active = "NO"
		ui.element.delete_button.active = "NO"
	end
	
	iup.Refresh(ui.dialog)
end

function ui.element.close_button:action() ui:Hide() end


-- Name Preset Dialog --

function ui.MakeRenameDialog(ok_cb, isDefault)
	local rename_element = {}
	rename_element.input_text = iup.text{value="", size="400x", nc=20}
	rename_element.error_label = iup.label{title="", size="400", fgcolor="255 0 0"}
	rename_element.default_toggle = iup.stationtoggle{title="Save as universe-wide default", value="OFF"}
	rename_element.ok_button = iup.stationbutton{title="Save"}
	rename_element.cancel_button = iup.stationbutton{title="Cancel"}
	
	if isDefault~=nil then
		rename_element.default_toggle.value = isDefault and "ON" or "OFF"
		rename_element.default_toggle.active = "NO"
	end
	
	local rename_frame = iup.vbox{
		iup.hbox{iup.fill{}, iup.label{title="Choose a name for the preset",font=Font.H3}, iup.fill{}},
		rename_element.input_text,
		rename_element.error_label,
		rename_element.default_toggle,
		iup.hbox{rename_element.cancel_button, iup.fill{}, rename_element.ok_button},
		alignment="ALEFT",
	}
	
	local dlg = iup.dialog{
		iup.vbox{
			iup.fill{},
			iup.hbox{
				iup.fill{},
				iup.pdarootframe{
				--iup.stationsubframe{
					rename_frame,
					expand="NO",
					gap=8,
				},
				iup.fill{},
			},
			iup.fill{},
		},
		BGCOLOR = "0 0 0 160 *",
		FULLSCREEN="YES",
		RESIZE="NO",
		BORDER="NO",
		MENUBOX="YES",
		TOPMOST="YES",
		DEFAULTESC=rename_element.cancel_button,
		DEFAULTENTER=rename_element.ok_button,
	}
	dlg:map()
	
	function rename_element.default_toggle:action()
		if self.value=="ON" then
			rename_element.input_text.value = "DEFAULT"
			rename_element.input_text.active = "NO"
		else
			rename_element.input_text.value = ""
			rename_element.input_text.active = "YES"
		end
	end
	
	function rename_element.ok_button:action()
		local new_name = rename_element.input_text.value
		local isValid, error_msg = SLM.VerifyName(new_name)
		
		if rename_element.default_toggle.vale=="OFF" and not isValid then
			rename_element.error_label.title=error_msg
			iup.Refresh(dlg)
			
			return
		end
		
		ok_cb(new_name)
		HideDialog(dlg)
	end

	function rename_element.cancel_button:action()
		HideDialog(dlg)
	end
	
	return dlg
end

--* END UI *--


function ui.LoadPresets()
	ui.element.preset_list[1] = nil --clear preset list
	
	--populate preset list contents
	ui.PushListItem(ui.element.preset_list, '\127ffffff'.."ACTIVE SHIP")
	
	--create sorted list of preset names
	local sorted_order = {} --sorted list of all loadouts
	for name,loadout in pairs(SLM.data.presets) do
		local color = SLM.GetLoadoutColor(loadout) or '\127ffffff'
		table.insert(sorted_order, color..name)
	end
	table.sort(sorted_order, function(a,b) return filter_colorcodes(a):lower() < filter_colorcodes(b):lower() end)
	
	for _,name in ipairs(sorted_order) do ui.PushListItem(ui.element.preset_list, name) end --add items from sorted list
	
	for i=1,4 do ui.PushListItem(ui.element.preset_list, '\127ffffff'.."DEFAULT "..i) end
	
	--populate station text
	if PlayerInStation() then --TODO: safety check to verify not in capship?
		local stationid = GetStationLocation()
		local station_presets = SLM.data.stations[stationid] and SLM.data.stations[stationid] or {}
		
		ui.element.sector_label.title = ShortLocationStr(GetCurrentSectorid())..": "
		ui.element.station_label.title = GetStationName(stationid)
		
		ui.element.buy_button.active = "YES"
		
		for i=1,4 do
			if station_presets[i] then
				local color = SLM.GetLoadoutColor(SLM.data.presets[ station_presets[i] ]) or ''
				ui.element.preset_labels[i].title = color..station_presets[i]
			else ui.element.preset_labels[i].title = "UNASSIGNED" end
			
			local default_loadout = ShipPresets[i]
			if default_loadout then
				ui.element.default_labels[i].title = ui.DEFAULTS_COLOR..SLM.GetTruncatedShipName(default_loadout)
			else ui.element.default_labels[i].title = ui.DEFAULTS_COLOR.."UNASSIGNED" end
			
			ui.element.preset_buttons[i].active = "YES"
			--ui.element.default_buttons[i].active = "YES"
		end
	else
		ui.element.sector_label.title = ShortLocationStr(GetCurrentSectorid())..": "
		ui.element.station_label.title = "NOT DOCKED AT STATION"
		
		ui.element.buy_button.active = "NO"
		
		for i=1,4 do
			ui.element.preset_labels[i].title = ""
			ui.element.preset_buttons[i].active = "NO"
			--ui.element.default_buttons[i].active = "NO"
		end
	end
	
	ui.element.preset_list.value = 1 --select first item
	--ui.element.preset_list:action(ui.element.preset_list[1], 1, 1) --simulate clicking list item
	ui.UpdateListSelection() --update loadout info panel
end

function ui.GetListSelected(list)
	local num = tostring(iup.GetAttribute(list, "VO_LISTSEL"))
	local val = iup.GetAttribute(list, num)
	
	return val
end

--Accurately determines what the VALUE attribute of a list should be.
function ui.GetListSize(list)
	local val = 0
	while list[val + 1] do val=val+1 end
	return val
end

--Adds an item to the end of a iup list
function ui.PushListItem(list, item)
	list.value = list.value or 0
	
	local new_val = list.value+1
	list[tostring(new_val)] = item
	--list[tostring(list.value)] = item
	list.value = new_val
	
	--list.value = list.value or 0
	--list['insertitem'..list.value+1] = item
end

--Removes the selected item from a list, returns the value of the item removed
function ui.RemoveListItem(list, sel)
	local size = ui.GetListSize(list)
	sel = sel or list.vo_listsel or size
	
	if sel~=0 then
		local item = list[tostring(sel)]
		
		while tonumber(sel) < size do
			list[sel] = list[sel+1]
			sel = sel+1
		end
		
		list[tostring(size)] = nil
		list.value = size-1
		list.vo_listsel = sel-1
		
		return item
	end
end


return ui --, vars